@@ -23,6 +23,12 @@
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#endif /* HAVE_MALLOC_H */
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif /* HAVE_STDLIB_H */
 #ifdef HAVE_DMALLOC_H
 #include <dmalloc.h>
 #endif /* HAVE_DMALLOC_H */
@@ -260,7 +266,7 @@ static cookies_entry_t* cookies_parse(cookies_t* cookies, uri_t* url_object, cha
   }
   for(i = 1; i < count; i++) {
     char* p = splitted[i];
-    while(*p && isspace(*p))
+    while(*p && isspace((int)*p))
       p++;
     if(!strncasecmp(p, "path", 4)) {
       char* equal = strchr(p, '=');
@@ -269,13 +275,13 @@ static cookies_entry_t* cookies_parse(cookies_t* cookies, uri_t* url_object, cha
 	/*
 	 * Strip leading space and /.
 	 */
-	while(*path && (isspace(*path) || *path == '/'))
+	while(*path && (isspace((int)*path) || *path == '/'))
 	  path++;
 	p = path;
 	/*
 	 * Strip trailing space.
 	 */
-	while(*p && !isspace(*p))
+	while(*p && !isspace((int)*p))
 	  p++;
 	*p = '\0';
       }
