@@ -53,7 +53,6 @@
 #include <ftp.h>
 #include <mime.h>
 #include <dirsel.h>
-#include <bodyparse.h>
 #ifdef LANGREC
 #include <decodage_usr.h>
 #endif /* LANGREC */
@@ -295,6 +294,7 @@ static void crawl_rebuild_1(char* paramsp, webbase_url_t* webbase_url)
     char query[256];
     char* path = uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH);
     if(file_exists(path)) {
+      hook_server(robots_url2server_id(params->base, webbase_url->w_url));
       hook_insert(params->base, webbase_url);
       if(verbose > 1) fprintf(stderr, "\trebuild %s (%d)\n", webbase_url->w_url, webbase_url->w_hookid);
       sprintf(query, "update url set hookid = %d where rowid = %d\n", webbase_url->w_hookid, webbase_url->w_rowid);
@@ -426,7 +426,7 @@ static void hp_remove_unloaded_start(crawl_params_t* params, webbase_url_start_t
 void hp_remove_unloaded(crawl_params_t* params, char* url)
 {
   char* where = smalloc(128 + strlen(url));
-  sprintf(where, "where url = '%s'", sql_quote_char_simple(url));
+  sprintf(where, "where url_md5 = '%.*s'", MD5_ASCII_SIZE, str2md5ascii_simple(url, strlen(url)));
   hp_action(params, where, hp_remove_unloaded_start, hp_remove_unloaded_1);
   free(where);
 }
@@ -434,6 +434,7 @@ void hp_remove_unloaded(crawl_params_t* params, char* url)
 webbase_url_t* crawl_touch(crawl_params_t* params, char* url)
 {
   crawl_context_t context;
+  webbase_url_t* webbase_url;
 
   memset(&context, '\0', sizeof(crawl_context_t));
   context.params = params;
@@ -441,12 +442,17 @@ webbase_url_t* crawl_touch(crawl_params_t* params, char* url)
   context.touch = 1;
   context.walk_end = 1; 
 
-  return mirror(&context, url);
+  webbase_url = mirror(&context, url);
+
+  webbase_url_free(&context.webbase_url);
+
+  return webbase_url;
 }
 
 void hp_load_in_core(crawl_params_t* params, char* url)
 {
   int url_length = strlen(url);
+  unsigned char url_md5[MD5_ASCII_SIZE];
   int exists;
   webbase_t* base = params->base;
   webbase_url_start_t* start_in_core = &params->start_in_core;
@@ -457,11 +463,14 @@ void hp_load_in_core(crawl_params_t* params, char* url)
     return;
   }
 
-  webbase_enable_start(base, url);
-  if((exists = (int)webbase_get_start(base, url, &start)) == 0) {
+  str2md5ascii(url, url_length, url_md5);
+
+  webbase_enable_start(base, url_md5);
+  if((exists = (int)webbase_get_start(base, url_md5, &start)) == 0) {
     webbase_start_reset(&start);
     webbase_default_start(base, &start);
     strcpy(start.url, url);
+    memcpy(start.url_md5, url_md5, MD5_ASCII_SIZE);
   }
 
   webbase_merge_start(&start, start_in_core);
@@ -522,8 +531,8 @@ static void hp_action_2(char* action_argp, MYSQL_RES* res, MYSQL_ROW row)
 {
   action_arg_t* action_arg = (action_arg_t*)action_argp;
   static webbase_url_t webbase_url_object;
-  char* url = row[0];
-  webbase_url_t* webbase_url = webbase_get_url(action_arg->params->base, url, &webbase_url_object, WEBBASE_GET_URL_ALL);
+  char* url_md5 = row[0];
+  webbase_url_t* webbase_url = webbase_get_url(action_arg->params->base, url_md5, &webbase_url_object, WEBBASE_GET_URL_ALL);
   if(webbase_url)
     (*action_arg->func)(action_arg->params, action_arg->start, webbase_url);
 }
@@ -539,7 +548,7 @@ static void hp_action_1(char* action_argp, webbase_url_start_t* start)
   
   {
     char query[128];
-    sprintf(query, "select url.url from start2url,url where start2url.start = %d and start2url.url = url.rowid", start->rowid);
+    sprintf(query, "select url.url_md5 from start2url,url where start2url.start = %d and start2url.url = url.rowid", start->rowid);
     sql_select(&params->base->mysql, 0, query, hp_action_2, (char*)action_arg);
   }
   
@@ -560,6 +569,7 @@ void hp_action(crawl_params_t* params, char* where, hp_action_start_callback fun
 void hp_unload(crawl_params_t* params, char* url, int keep_start)
 {
   int url_length = strlen(url);
+  unsigned char url_md5[MD5_ASCII_SIZE];
   webbase_t* base = params->base;
   static webbase_url_start_t start;
   crawl_context_t context;
@@ -569,8 +579,10 @@ void hp_unload(crawl_params_t* params, char* url, int keep_start)
     return;
   }
 
-  webbase_enable_start(base, url);
-  if(webbase_get_start(base, url, &start) == 0) {
+  str2md5ascii(url, url_length, url_md5);
+
+  webbase_enable_start(base, url_md5);
+  if(webbase_get_start(base, url_md5, &start) == 0) {
     fprintf(stderr, "cannot unload %s because it does not exist in start\n", url);
     return;
   }
@@ -600,6 +612,8 @@ void hp_unload(crawl_params_t* params, char* url, int keep_start)
     smysql_query(&base->mysql, query);
   }
 
+  webbase_url_free(&context.webbase_url);
+
 #ifdef WEBBASE_LOCK
   webbase_unlock(base, start.rowid, "start");
 #endif /* WEBBASE_LOCK */
@@ -676,6 +690,11 @@ static void hp_load_mirror(char* paramsp, webbase_url_start_t* start)
   crawl_params_t* params = (crawl_params_t*)paramsp;
   crawl_context_t context;
 
+  if(!webbase_url_start_ok(start->url, strlen(start->url))) {
+    fprintf(stderr, "hp_load_mirror: %s is not a valid starting point\n", start->url);
+    return;
+  }
+  
   memset(&context, '\0', sizeof(crawl_context_t));
   context.params = params;
   context.base = params->base;
@@ -724,6 +743,7 @@ static void hp_load_mirror(char* paramsp, webbase_url_start_t* start)
     params->robot_delayed++;
   free(context.home);
   free(context.dirsel_key);
+  webbase_url_free(&context.webbase_url);
 }
 
 /*
@@ -844,11 +864,17 @@ static int internal_bootstrap_start2url(webbase_t* base, char* query, char* url,
   MYSQL_ROW row;
   int url_rowid;
   int ret = 0;
+  unsigned char url_md5[MD5_ASCII_SIZE];
 
-  url = sql_quote_char_simple(url);
-  sprintf(query, "select rowid,url from url where url = '%s'", url);
+  str2md5ascii(url, strlen(url), url_md5);
+
+  sprintf(query, "select rowid from url where url_md5 = '%.*s'", MD5_ASCII_SIZE, url_md5);
   if(verbose > 1) fprintf(stderr, "\tbootstrap_start2url: query = %s\n", query);
-  if(webbase_get_url_2(base, query, url, &res, &row)) {
+
+  smysql_query(&base->mysql, query);
+  res = smysql_store_result(&base->mysql);
+  if(mysql_num_rows(res)) {
+    row = mysql_fetch_row(res);
     url_rowid = atoi(row[0]);
     webbase_insert_start2url(base, start_rowid, url_rowid, level);
   } else {
@@ -1057,6 +1083,7 @@ static int mirror_query(crawl_context_t* context, char* query)
       char* url = row[0];
 
       robots_load(context->params->robots, context, url);
+      hook_server(robots_server_id(context->params->robots));
       if((webbase_url = mirror(context, url))) {
 	/*
 	 * walk_href returns the status of the last 'mirror_crawl_continue'
@@ -1096,10 +1123,14 @@ int mirror_collect(char* contextp, webbase_url_t* base_url, char* url, int flag)
 
   if(verbose > 1) fprintf(stderr, "\tmirror_collect: %s\n", url);
   if(flag == WEBBASE_URL_WALK_ROBOTS || mirror_crawl_test(context, url)) {
+    unsigned char url_md5[MD5_ASCII_SIZE];
+
+    str2md5ascii(url, strlen(url), url_md5);
+
 #ifdef WEBBASE_LOCK
     webbase_lock(base, 0, url);
 #endif /* WEBBASE_LOCK */
-    rowid = webbase_exists_url(base, url);
+    rowid = webbase_exists_url(base, url_md5);
 
     if(rowid) {
       int counted = webbase_counted(base, context->start, rowid);
@@ -1245,6 +1276,8 @@ static webbase_url_t* mirror_2(crawl_context_t* context, char* url)
   webbase_url_t* webbase_url = &webbase_url_object;
   static uri_t* url_object = 0;
   int url_length = strlen(url);
+  unsigned char url_md5[MD5_ASCII_SIZE];
+  
 
   /*
    * Sanity checks
@@ -1266,11 +1299,13 @@ static webbase_url_t* mirror_2(crawl_context_t* context, char* url)
     fprintf(stderr, "mirror: cannot crawl relative URL %s\n", url);
     return 0;
   }
-  
+
+  str2md5ascii(url, strlen(url), url_md5);
+
   /*
    * Retrieve all available information about the url
    */
-  webbase_get_url(context->base, url, webbase_url, WEBBASE_GET_URL_ALL);
+  webbase_get_url(context->base, url_md5, webbase_url, WEBBASE_GET_URL_ALL);
   if(webbase_url->w_rowid == 0) {
     fprintf(stderr, "mirror_2: could not find %s in url table\n", url);
     return 0;
@@ -1409,15 +1444,20 @@ static void mirror_scheme(crawl_context_t* context, webbase_url_t* webbase_url,
       webbase_url->w_info |= WEBBASE_URL_INFO_TIMEOUT;
       break;
     case ETIMEDOUT:
+#ifdef ETIME
     case ETIME:
+#endif /* ETIME */
       context->timeouts++;
       webbase_url->w_code = WEBBASE_URL_CODE_CONNECTION_TIMED_OUT;
       webbase_url->w_info |= WEBBASE_URL_INFO_TIMEOUT;
       break;
+    case E2BIG:
+      webbase_url->w_info |= WEBBASE_URL_INFO_TRUNCATED;
+      break;
     default:
       /* Unknown non fatal error, consider it timeout but print it. */
-      fprintf(stderr, "mirror: %s failed because ", webbase_url->w_url);
       perror("");
+      fprintf(stderr, "mirror: %s failed\n", webbase_url->w_url);
       webbase_url->w_code = WEBBASE_URL_CODE_CONNECTION_TIMED_OUT;
       webbase_url->w_info |= WEBBASE_URL_INFO_TIMEOUT;
       break;
@@ -1447,10 +1487,10 @@ static void ftp_body_parse(webbase_url_t* webbase_url, uri_t* url_object, char*
       char* to = buffer;
       *to++ = *from++;
       while(from - buffer < buffer_length) {
-	if(to[-1] == ' ' && isspace(*from)) {
+	if(to[-1] == ' ' && isspace((int)*from)) {
 	    from++;
 	} else {
-	  if(isspace(*from)) {
+	  if(isspace((int)*from)) {
 	    *to++ = ' ';
 	    from++;
 	  } else {
@@ -1583,7 +1623,7 @@ static int mirror_ftp(crawl_context_t* context, webbase_url_t* webbase_url, uri_
 	} else {
 	    fprintf(stderr, "mirror_ftp: cannot cache %s, unable to resolve path (ignored)\n", webbase_url->w_url);
 	    webbase_url_code_set(webbase_url, WEBBASE_URL_CODE_BAD_REQUEST);
-	    return ETIME;
+	    return ETIMEDOUT;
 	}
 
 	isdir = (url[url_length - 1] == '/' || url[0] == '\0') ? 1 : 0;
@@ -1596,7 +1636,7 @@ static int mirror_ftp(crawl_context_t* context, webbase_url_t* webbase_url, uri_
 	if(!code) {
 	    fprintf(stderr, "mirror_ftp: %s cannot get/ls `%s'\n", url, url_object->path);
 	    webbase_url_code_set(webbase_url, WEBBASE_URL_CODE_NOT_FOUND);
-	    return ETIME;
+	    return ETIMEDOUT;
 	}
 
 	webbase_url_code_set(webbase_url, WEBBASE_URL_CODE_OK);
@@ -1674,33 +1714,35 @@ static int mirror_http(crawl_context_t* context, webbase_url_t* webbase_url, uri
       if(WEBBASE_URL_INFO_LOAD_BODY(webbase_url->w_code) && mime_accept_type(context->base, spec, webbase_url->w_content_type)) {
 	webbase_url->w_info &= ~WEBBASE_URL_INFO_READING;
 	http->mode = WEBTOOLS_READER_HTTP_BODY;
-	if(webtools_reader(http, sd) == WEBTOOLS_READER_OK) {
+	if((error = webtools_reader(http, sd)) == WEBTOOLS_READER_OK ||
+	   error == WEBTOOLS_READER_TRUNC) {
 	  if(verbose > 1) fprintf(stderr, "\tloading %s : body read\n", url);
 	  webbase_url_hrefs_set(webbase_url, webbase_url->path, context->start->size_limit);
+#ifdef LANGREC
 	  {
 	    char* raw_path = webbase_url->path;
 	    char* path = smalloc(strlen(raw_path) + 2);
 	    sprintf(path, "%sR", raw_path);
-	    if(bodyparse(raw_path, path, webbase_url->w_url, 100000000, 0, BODY_PARSE_UNACCENT) < 0) {
+	    if(bodyparse(raw_path, path, webbase_url->w_url, 1000000, 0, BODY_PARSE_UNACCENT) < 0) {
 	      fprintf(stderr, "mirror_http: cannot convert to text %s\n", raw_path);
 	    } else {
-#ifdef LANGREC
 	      char language[DECODAGE_TAILLE_MAX];
 	      textlang_in_file(path, language);
 	      memcpy(webbase_url->w_language, language, WEBBASE_LANGUAGE_LENGTH);
 	      webbase_url->w_language[WEBBASE_LANGUAGE_LENGTH] = '\0';
 	      if(verbose > 1) fprintf(stderr, "\tlanguage = %s\n", webbase_url->w_language);
-#else /* LANGREC */
 	      webbase_url->w_language[0] = '\0';
-#endif /* LANGREC */
 	      unlink(path);
 	    }
 	    free(path);
 	  }
+#endif /* LANGREC */
 	  webbase_url_content_length_fix(webbase_url, webbase_url->path);
 	  webbase_url_mtime_fix(webbase_url);
 	  webbase_url->path = 0;
 	  webbase_url->w_info &= ~(WEBBASE_URL_INFO_TRUNCATED | WEBBASE_URL_INFO_READING);
+	  if(error == WEBTOOLS_READER_TRUNC)
+	    error = E2BIG;
 	} else {
 	  webbase_url->w_info |= WEBBASE_URL_INFO_TRUNCATED;
 	  error = errno;
@@ -1884,6 +1926,7 @@ static void mirror_schedule(crawl_context_t* context, webbase_url_t* webbase_url
   } else if(webbase_url->w_info & WEBBASE_URL_INFO_ERROR) {
     delay = start->not_found_delay * WEBBASE_TIME_DAY;
   } 
+  if(delay <= 0) delay = WEBBASE_TIME_WEEK;
 
   webbase_url->w_crawl = time(0) + delay;
 }
@@ -1899,6 +1942,7 @@ void crawl_free(crawl_params_t* params)
   webbase_free(params->base);
   webtools_free(params->http);
   mirror_ftp_free();
+  webbase_start_free(&params->start_in_core);
   free(params);
 }
 
