@@ -23,6 +23,12 @@
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#endif /* HAVE_MALLOC_H */
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif /* HAVE_STDLIB_H */
 #ifdef HAVE_DMALLOC_H
 #include <dmalloc.h>
 #endif /* HAVE_DMALLOC_H */
