@@ -41,7 +41,10 @@
 #include <dirsel.h>
 
 static void robots_decode(robots_t* robots, MYSQL_RES *res, MYSQL_ROW row);
-static void robots_load_1(robots_t* robots, crawl_context_t* context, uri_t* url_object);
+/*
+ * Returns 0 if ok, -1 if should try again.
+ */
+static int robots_load_1(robots_t* robots, crawl_context_t* context, uri_t* url_object);
 static void robots_save_entry(robots_t* robots);
 static void robots_reset_entry(robots_t* robots);
 static robots_t* params_alloc();
@@ -133,11 +136,55 @@ void robots_load(robots_t* robots, crawl_context_t* context, char* url)
       if(verbose) fprintf(stderr, "robots_load: reuse current %s\n", netloc);
     } else {
       if(verbose) fprintf(stderr, "robots_load: load %s\n", netloc);
-      robots_load_1(robots, context, url_object);
+      /*
+       * If robots entry fails to be inserted, it means that between the
+       * time we checked if the entry existed and the time of the insertion,
+       * another process did the insertion for us. Just reexecute, the record
+       * will be found and everyone will be happy.
+       */
+      if(robots_load_1(robots, context, url_object) < 0)
+	robots_load_1(robots, context, url_object);
     }
   }
 }
 
+int robots_url2server_id(webbase_t* base, char* url)
+{
+  static uri_t* url_object = 0;
+  char* netloc;
+  int url_length = strlen(url);
+  int server_id = 0;
+
+  /*
+   * Build an url object from url
+   */
+  if(!url_object) {
+    url_object = uri_alloc(url, url_length);
+  } else {
+    uri_realloc(url_object, url, url_length);
+  }
+
+  if(!strncasecmp(uri_scheme(url_object), "http", 4)) {
+    char server_id_string[32];
+    char query[512];
+
+    netloc = uri_netloc(url_object);
+
+    sprintf(query, "select rowid from robots where netloc = '%s'", sql_quote_char_simple(netloc, strlen(netloc)));
+    if(sql_select_value(&base->mysql, query, server_id_string, 32))
+      server_id = atoi(server_id_string);
+  }
+
+  return server_id;
+}
+
+int robots_server_id(robots_t* robots)
+{
+  robots_entry_t* entry = &robots->current;
+
+  return entry->rowid;
+}
+
 void robots_info(robots_t* robots, int robot_delay, uri_t* url_object, time_t* delay, int* delayed)
 {
   time_t now = time(0);
@@ -160,7 +207,7 @@ void robots_info(robots_t* robots, int robot_delay, uri_t* url_object, time_t* d
   if(verbose) fprintf(stderr, "robots_info: for %s => delayed = %s, delay = %ld\n", uri_uri(url_object), (*delayed ? "yes" : "no"), *delay);
 }
 
-static void robots_load_1(robots_t* robots, crawl_context_t* context, uri_t* url_object)
+static int robots_load_1(robots_t* robots, crawl_context_t* context, uri_t* url_object)
 {
   static char* query = 0;
   static int query_size = 0;
@@ -207,14 +254,22 @@ static void robots_load_1(robots_t* robots, crawl_context_t* context, uri_t* url
 
     static_alloc(&query, &query_size, 128 + netloc_length + strlen(allow) + strlen(disallow));
 
-    sprintf(query, "replace into robots values ('%s', %d, %d, %s, %s)",
+    sprintf(query, "replace into robots values (0, '%s', %d, %d, %s, %s)",
 	    netloc,
 	    robots->current.next_crawl,
 	    robots->current.queue_crawl,
 	    allow,
 	    disallow);
     if(verbose) fprintf(stderr, "robots_load: query = %s\n", query);
-    smysql_query(&base->mysql, query);
+    if(mysql_query(&base->mysql, query)) {
+      if(mysql_errno(&base->mysql) != ER_DUP_ENTRY) {
+	fprintf(stderr, "%s: %s\n", query, mysql_error(&base->mysql));
+	exit(1);
+      } else {
+	free(netloc);
+	return -1;
+      }
+    }
 
     /*
        * Query the base to get the information just inserted. This
@@ -232,7 +287,6 @@ static void robots_load_1(robots_t* robots, crawl_context_t* context, uri_t* url
   }
 
   {
-    char* netloc = strdup(uri_netloc(url_object));
     if(robots->current.disallow && robots->current.disallow[0] != '\0')
       dirsel_robots_disallow(netloc, robots->current.disallow, DIRSEL_LOAD);
     if(robots->current.allow && robots->current.allow[0] != '\0')
@@ -241,12 +295,15 @@ static void robots_load_1(robots_t* robots, crawl_context_t* context, uri_t* url
 
   mysql_free_result(res);
   free(netloc);
+
+  return 0;
 }
 
 static void robots_reset_entry(robots_t* robots)
 {
   robots_entry_t* entry = &robots->current;
 
+  entry->rowid = 0;
   entry->info = 0;
   entry->netloc[0] = '\0';
   if(entry->allow)
@@ -285,16 +342,17 @@ static void robots_decode(robots_t* robots, MYSQL_RES *res, MYSQL_ROW row)
 
   robots_reset_entry(robots);
 
-  strcpy(entry->netloc, row[0]);
-  entry->next_crawl = atoi(row[1]);
-  entry->queue_crawl = atoi(row[2]);
-  if(row[3]) {
-    static_alloc(&entry->allow, &entry->allow_size, strlen(row[3]) + 1);
-    strcpy(entry->allow, row[3]);
-  }
+  entry->rowid = atoi(row[0]);
+  strcpy(entry->netloc, row[1]);
+  entry->next_crawl = atoi(row[2]);
+  entry->queue_crawl = atoi(row[3]);
   if(row[4]) {
-    static_alloc(&entry->disallow, &entry->disallow_size, strlen(row[4]) + 1);
-    strcpy(entry->disallow, row[4]);
+    static_alloc(&entry->allow, &entry->allow_size, strlen(row[4]) + 1);
+    strcpy(entry->allow, row[4]);
+  }
+  if(row[5]) {
+    static_alloc(&entry->disallow, &entry->disallow_size, strlen(row[5]) + 1);
+    strcpy(entry->disallow, row[5]);
   }
   entry->info |= ROBOTS_INFO_USED;
 
