@@ -59,6 +59,7 @@ void smysql_queries(MYSQL* mysql, const char** queries, int queries_length)
 int smysql_query(MYSQL* mysql, const char* query)
 {
   int ret;
+  if(verbose > 1) fprintf(stderr, "smysql_query: %s\n", query);
   if((ret = mysql_query(mysql, query))) {
     fprintf(stderr, "%s: %s\n", query, mysql_error(mysql));
     exit(1);
@@ -78,38 +79,39 @@ MYSQL_RES* smysql_store_result(MYSQL* mysql)
   return res;
 }
 
-char* sql_quote_char_simple(char* from)
+char* sql_quote_char_simple(char* from, int from_size)
 {
   static char* to = 0;
   static int to_size = 0;
 
-  return sql_quote_char(&to, &to_size, from);
+  return sql_quote_char(&to, &to_size, from, from_size);
 }
 
-char* sql_quote_char(char** to, int* to_size, char* from)
+/*
+ * Quote special characters ' \ " and null char.
+ */
+char* sql_quote_char(char** to, int* to_size, char* from, int from_size)
 {
-  static char* sfrom[] = {
-    "\\",
-    "'",
-    "\"",
-    0
-  };
-  static char* sto[] = {
-    "\\\\",
-    "''",
-    "\\\"",
-    0
-  };
+  int i, j;
 
-  int tmp_size;
-  char* tmp;
+  static_alloc(to, to_size, from_size + 1);
 
-  if(!from) from = "";
+  for(i = 0, j = 0; i < from_size; i++, j++) {
+    char c = from[i];
+    switch(c) {
+    case '\0':
+      c = '0';
+    case '\\':
+    case '\'':
+    case '"':
+      static_alloc(to, to_size, *to_size + 1);
+      (*to)[j++] = '\\';
+      break;
+    }
+    (*to)[j] = c;
+  }
 
-  tmp = strsubs(from, strlen(from), &tmp_size, sfrom, sto);
-
-  static_alloc(to, to_size, tmp_size + 1);
-  strcpy(*to, tmp);
+  (*to)[j] = '\0';
 
   return *to;
 }
@@ -151,6 +153,8 @@ int sql_select_value(MYSQL* mysql, char* query, char* value, int value_size)
   MYSQL_ROW row;
   int found = 0;
 
+  if(verbose > 1) fprintf(stderr, "sql_select_value: %s\n", query);
+
   smysql_query(mysql, query);
   res = smysql_store_result(mysql);
 
@@ -175,6 +179,7 @@ int sql_select(MYSQL* mysql, char* header, char* query, sql_select_callback call
   int num_rows;
 
   if(verbose) time_register(0);
+  if(verbose > 1) fprintf(stderr, "sql_select: %s\n", query);
   smysql_query(mysql, query);
   res = smysql_store_result(mysql);
   if(verbose) time_show(query, 0);
