@@ -42,12 +42,13 @@
 #include <getopttools.h>
 #include <webbase_url.h>
 #include <sqlutil.h>
+#include <md5str.h>
 
 static int verbose = 0;
 
 static webbase_url_start_t* webbase_decode_start(webbase_t* base, MYSQL_RES *res, MYSQL_ROW row, webbase_url_start_t* start);
 static webbase_url_t* webbase_decode_url(webbase_t* base, MYSQL_RES *res, MYSQL_ROW row, webbase_url_t* webbase_url);
-static webbase_url_start_t* webbase_get_start_1(webbase_t* base, char* quote, char* field, char* value, webbase_url_start_t* start);
+static webbase_url_start_t* webbase_get_start_1(webbase_t* base, char* quote, unsigned char* field, char* value, int value_length, webbase_url_start_t* start);
 static webbase_t* webbase_alloc_1();
 
 static struct option long_options[] =
@@ -164,28 +165,34 @@ webbase_t* webbase_alloc(int argc, char** argv, struct option options[])
    * Open database
    */
   {
-    int i;
-    int error = 0;
-
-    if(base->mysql_port != 0) mysql_port = base->mysql_port;
     if(base->mysql_host[0] == '\0') strcpy(base->mysql_host, "localhost");
-    if(base->mysql_unix_port != 0) mysql_unix_port = base->mysql_unix_port;
 
-    for(i = 0; i < 2; i++) {
-      if(!(mysql_connect(&base->mysql, base->mysql_host, user, password))) {
-	fprintf(stderr, "webbase_alloc: connect: %s\n", mysql_error(&base->mysql));
-	error++;
-	/*
-	 * Connection may fail if the server is overloaded, give it
-	 * a rest.
-	 */
-	sleep(3);
-      } else {
-	error = 0;
-	break;
+    /*
+     * Initialize structure and set defaults
+     */
+    mysql_init(&base->mysql);
+    /*
+     * Instruct real_connect to read missing parameters from ~/.my.cnf file.
+     */
+    {
+      char* home = getenv("HOME");
+      if(home) {
+	char* cnf_file = (char*)smalloc(strlen(home) + 32);
+	sprintf(cnf_file, "%s/.my.cnf", home);
+	mysql_options(&base->mysql, MYSQL_READ_DEFAULT_FILE, cnf_file);
+	free(cnf_file);
       }
     }
-    if(error) {
+    
+    if(!(mysql_real_connect(&base->mysql,
+			    base->mysql_host[0] != '\0' ? base->mysql_host : (char*)0,
+			    user[0] != '\0' ? user : (char*)0,
+			    password[0] != '\0' ? password : (char*)0,
+			    (char*)0,
+			    base->mysql_port,
+			    base->mysql_unix_port,
+			    0))) {
+      fprintf(stderr, "webbase_alloc: connect: %s\n", mysql_error(&base->mysql));
       fprintf(stderr, "webbase_alloc: connect: failed to connect\n");
       exit(1);
     }
@@ -244,15 +251,25 @@ void webbase_lock_ignore(webbase_t* base, int varint, char* varchar)
   }
 }
 
+/*
+ * Insert a lock in the lock table. The name of the lock is
+ * either a concatenation of varing and varchar argument (name), if
+ * their cumulated size is lower than MD5_ASCII_SIZE or the
+ * MD5 key calculated on (name).
+ */
 int webbase_lock(webbase_t* base, int varint, char* varchar)
 {
   char query[1024];
   int locked = 0;
   int loop_count = 0;
   char name[512];
+  int name_length;
+  unsigned char* md5;
+  int md5_length;
 
   sprintf(name, "%d%s", varint, varchar);
-
+  name_length = strlen(name);
+  
   {
     char* ignore = getenv("ECILA_IGNORE");
     if(ignore && !strcmp(name, ignore)) {
@@ -261,7 +278,15 @@ int webbase_lock(webbase_t* base, int varint, char* varchar)
     }
   }
 
-  sprintf(query, "insert into locks values ('%s', '%d')", sql_quote_char_simple(name), base->pid);
+  if(name_length <= MD5_ASCII_SIZE) {
+    md5 = name;
+    md5_length = name_length;
+  } else {
+    md5 = str2md5ascii_simple(name, strlen(name));
+    md5_length = MD5_ASCII_SIZE;
+  }
+
+  sprintf(query, "insert low_priority into locks values ('%.*s', '%d')", md5_length, md5, base->pid);
 
   if(verbose) fprintf(stderr, "\tlocking %s ...", name);
   while(locked == 0) {
@@ -278,7 +303,7 @@ int webbase_lock(webbase_t* base, int varint, char* varchar)
 	exit(1);
       }
       if(verbose) fprintf(stderr, " check pid for %s ", name);
-      sprintf(query2, "select pid from locks where name = '%s'", sql_quote_char_simple(name));
+      sprintf(query2, "select high_priority pid from locks where name = '%.*s'", md5_length, md5);
       if(sql_select_value(&base->mysql, query2, pid_string, 32) == 0) {
 	/*
 	 * Lock is gone
@@ -293,7 +318,7 @@ int webbase_lock(webbase_t* base, int varint, char* varchar)
 	} else {
 	  if(kill(pid, 0) < 0) {
 	    if(verbose) fprintf(stderr, " process died, cleanup ... ");
-	    sprintf(query2, "delete from locks where name = '%s'", sql_quote_char_simple(name));
+	    sprintf(query2, "delete low_priority from locks where name = '%.*s'", md5_length, md5);
 	    smysql_query(&base->mysql, query2);
 	  } else {
 	    if(verbose) fprintf(stderr, " sleep %d ... ", base->lock_wait);
@@ -313,8 +338,12 @@ void webbase_unlock(webbase_t* base, int varint, char* varchar)
 {
   char query[1024];
   char name[512];
+  int name_length;
+  unsigned char* md5;
+  int md5_length;
 
   sprintf(name, "%d%s", varint, varchar);
+  name_length = strlen(name);
 
   {
     char* ignore = getenv("ECILA_IGNORE");
@@ -324,14 +353,22 @@ void webbase_unlock(webbase_t* base, int varint, char* varchar)
     }
   }
 
-  sprintf(query, "delete from locks where name = '%s' and pid = %d", sql_quote_char_simple(name), base->pid);
+  if(name_length <= MD5_ASCII_SIZE) {
+    md5 = name;
+    md5_length = name_length;
+  } else {
+    md5 = str2md5ascii_simple(name, strlen(name));
+    md5_length = MD5_ASCII_SIZE;
+  }
+
+  sprintf(query, "delete low_priority from locks where name = '%.*s' and pid = %d", md5_length, md5, base->pid);
 
   if(verbose) fprintf(stderr, "\tunlocking %s ...", name);
   smysql_query(&base->mysql, query);
   if(mysql_affected_rows(&base->mysql) == 0) {
     char pid_string[32];
     if(verbose) fprintf(stderr, "not found ");
-    sprintf(query, "select pid from locks where name = '%s'", sql_quote_char_simple(name));
+    sprintf(query, "select high_priority pid from locks where name = '%.*s'", md5_length, md5);
     if(sql_select_value(&base->mysql, query, pid_string, 32) == 0) {
       if(verbose) fprintf(stderr, ", no row ");
     } else {
@@ -444,41 +481,39 @@ webbase_url_start_t* webbase_get_start_of_url(webbase_t* base, webbase_url_t* we
   return webbase_get_start_rowid(base, start_rowid, start);
 }
 
-void webbase_enable_start(webbase_t* base, char* url)
+void webbase_enable_start(webbase_t* base, unsigned char* url_md5)
 {
   static char* query = 0;
   static int query_size = 0;
-  int url_length = strlen(url);
 
-  static_alloc(&query, &query_size, 256 + url_length);
+  static_alloc(&query, &query_size, 256 + MD5_ASCII_SIZE);
 
-  sprintf(query, "update start set disabled = 'no' where url = '%s'", sql_quote_char_simple(url));
+  sprintf(query, "update start set disabled = 'no' where url_md5 = '%.*s'", MD5_ASCII_SIZE, url_md5);
   smysql_query(&base->mysql, query);
 }
 
-webbase_url_start_t* webbase_get_start(webbase_t* base, char* url, webbase_url_start_t* start)
+webbase_url_start_t* webbase_get_start(webbase_t* base, unsigned char* url_md5, webbase_url_start_t* start)
 {
-  return webbase_get_start_1(base, "'", "url", sql_quote_char_simple(url), start);
+  return webbase_get_start_1(base, "'", "url_md5", url_md5, MD5_ASCII_SIZE, start);
 }
 
 webbase_url_start_t* webbase_get_start_rowid(webbase_t* base, int rowid, webbase_url_start_t* start)
 {
-  char rowid_string[32];
+  unsigned char rowid_string[32];
   sprintf(rowid_string, "%d", rowid);
-  return webbase_get_start_1(base, "", "rowid", rowid_string, start);
+  return webbase_get_start_1(base, "", "rowid", rowid_string, strlen(rowid_string), start);
 }
 
-static webbase_url_start_t* webbase_get_start_1(webbase_t* base, char* quote, char* field, char* value, webbase_url_start_t* start)
+static webbase_url_start_t* webbase_get_start_1(webbase_t* base, char* quote, unsigned char* field, char* value, int value_length, webbase_url_start_t* start)
 {
   MYSQL_RES *res;
   MYSQL_ROW row;
   static char* query = 0;
   static int query_size = 0;
-  int value_length = strlen(value);
 
   static_alloc(&query, &query_size, 256 + value_length);
 
-  sprintf(query, "select " WEBBASE_START_FIELDS " from start where %s = %s%s%s and disabled = 'no'", field, quote, value, quote);
+  sprintf(query, "select " WEBBASE_START_FIELDS " from start where %s = %s%.*s%s and disabled = 'no'", field, quote, value_length, value, quote);
   smysql_query(&base->mysql, query);
   res = smysql_store_result(&base->mysql);
   if(mysql_num_rows(res) > 0) {
@@ -522,6 +557,8 @@ void webbase_update_start(webbase_t* base, webbase_url_start_t* start)
   static int hook_info_size = 0;
   static char* url = 0;
   static int url_size = 0;
+  static char* url_md5 = 0;
+  static int url_md5_size = 0;
   int has_auth = start->info & WEBBASE_URL_START_AUTH;
   int has_accept = start->info & WEBBASE_URL_START_ACCEPT;
   int has_filter = start->info & WEBBASE_URL_START_FILTER;
@@ -531,24 +568,35 @@ void webbase_update_start(webbase_t* base, webbase_url_start_t* start)
 
   webbase_url_start_t* default_start = &base->default_start;
 
-  if(has_auth)
-    sql_quote_char(&auth, &auth_size, start->auth);
-  if(has_accept)
-    sql_quote_char(&accept, &accept_size, start->accept);
-  if(has_filter)
-    sql_quote_char(&filter, &filter_size, start->filter);
-  if(has_allow)
-    sql_quote_char(&allow, &allow_size, start->allow);
-  if(has_disallow)
-    sql_quote_char(&disallow, &disallow_size, start->disallow);
-  if(has_hook_info)
-    sql_quote_char(&hook_info, &hook_info_size, start->hook_info);
+#define MINIMUM_QUERY_SIZE (20 * 1024)  
+  static_alloc(&query, &query_size, MINIMUM_QUERY_SIZE);
 
-  static_alloc(&query, &query_size, 10240 + auth_size + accept_size + filter_size + allow_size + disallow_size + hook_info_size);
+  if(has_auth)
+    sql_quote_char(&auth, &auth_size, start->auth, strlen(start->auth));
+  if(has_accept)
+    sql_quote_char(&accept, &accept_size, start->accept, strlen(start->accept));
+  if(has_filter)
+    sql_quote_char(&filter, &filter_size, start->filter, strlen(start->filter));
+  if(has_allow)
+    sql_quote_char(&allow, &allow_size, start->allow, strlen(start->allow));
+  if(has_disallow)
+    sql_quote_char(&disallow, &disallow_size, start->disallow, strlen(start->disallow));
+  if(has_hook_info)
+    sql_quote_char(&hook_info, &hook_info_size, start->hook_info, strlen(start->hook_info));
+
+  static_alloc(&query, &query_size, MINIMUM_QUERY_SIZE +
+	       url_size +
+	       auth_size +
+	       accept_size +
+	       filter_size +
+	       allow_size +
+	       disallow_size +
+	       hook_info_size);
 
   if(start->rowid != 0) {
-    sprintf(query, "update start set url = '%s', info = %d",
-	    sql_quote_char(&url, &url_size, start->url),
+    sprintf(query, "update start set url = '%s', url_md5 = '%s', info = %d",
+	    sql_quote_char(&url, &url_size, start->url, strlen(start->url)),
+	    sql_quote_char(&url_md5, &url_md5_size, start->url_md5, MD5_ASCII_SIZE),
 	    start->info);
 #define S(field) \
     strcat(query, ", " #field " = "); \
@@ -580,9 +628,10 @@ void webbase_update_start(webbase_t* base, webbase_url_start_t* start)
     sprintf(query + strlen(query), " where rowid = %d", start->rowid);
     if(verbose > 5) fprintf(stderr, "update start: %s\n", query);
   } else {
-    sprintf(query, "replace into start ("WEBBASE_START_FIELDS_INSERT") values (%d, '%s', %d, ",
+    sprintf(query, "replace into start ("WEBBASE_START_FIELDS_INSERT") values (%d, '%s', '%s', %d, ",
 	    start->rowid,
-	    sql_quote_char(&url, &url_size, start->url),
+	    sql_quote_char(&url, &url_size, start->url, strlen(start->url)),
+	    sql_quote_char(&url_md5, &url_md5_size, start->url_md5, MD5_ASCII_SIZE),
 	    start->info);
 #define S(field) \
     strcat(query, start->field == default_start->field ? "null" : sql_itoa(start->field)); \
@@ -619,6 +668,7 @@ void webbase_update_start(webbase_t* base, webbase_url_start_t* start)
   if(start->rowid == 0) {
     start->rowid = mysql_insert_id(&base->mysql);
   }
+#undef MINIMUM_QUERY_SIZE
 }
 
 void webbase_merge_start(webbase_url_start_t* to, webbase_url_start_t* from)
@@ -650,9 +700,20 @@ void webbase_merge_start(webbase_url_start_t* to, webbase_url_start_t* from)
 #undef S
 }
 
+void webbase_start_free(webbase_url_start_t* start)
+{
+  if(start->auth) free(start->auth);
+  if(start->accept) free(start->accept);
+  if(start->filter) free(start->filter);
+  if(start->allow) free(start->allow);
+  if(start->disallow) free(start->disallow);
+  if(start->hook_info) free(start->hook_info);
+}
+
 void webbase_start_reset(webbase_url_start_t* start)
 {
   start->url[0] = '\0';
+  memset(start->url_md5, '\0', sizeof(start->url_md5));
   start->info = 0;
   start->url_max_size = -1;
   start->size_limit = -1;
@@ -682,6 +743,10 @@ static webbase_url_start_t* webbase_decode_start(webbase_t* base, MYSQL_RES *res
   static webbase_url_start_t start_object;
   webbase_url_start_t* default_start = &base->default_start;
   int i;
+  /*
+   * Set if url_md5 field found and not empty.
+   */
+  int url_md5_set = 0;
 
   if(!start)
     start = &start_object;
@@ -735,9 +800,17 @@ static webbase_url_start_t* webbase_decode_start(webbase_t* base, MYSQL_RES *res
 #undef S
     } else if(!strcmp(field->name, "count")) {
       start->count = row[i] ? atoi(row[i]) : 0;
+    } else if(!strcmp(field->name, "url_md5")) {
+      if(row[i] && field->length == MD5_ASCII_SIZE) {
+	url_md5_set = 1;
+	memcpy(start->url_md5, row[i], sizeof(start->url_md5));
+      }
     }
   }
 
+  if(!url_md5_set)
+    str2md5ascii(start->url, strlen(start->url), start->url_md5);
+
   if(verbose > 1) fprintf(stderr, "decode_start: %s\n", start->url);
   return start;
 }
@@ -758,8 +831,9 @@ int webbase_insert_url(webbase_t* base, webbase_url_t* webbase_url)
   static char* query = 0;
   static int query_size = 0;
   int complete_rowid = 0;
-  
-  static_alloc(&query, &query_size, 10240);
+
+#define MINIMUM_QUERY_SIZE (20 * 1024)  
+  static_alloc(&query, &query_size, MINIMUM_QUERY_SIZE);
 
   if(webbase_url->w_info & WEBBASE_URL_INFO_COMPLETE) {
     /*
@@ -785,17 +859,20 @@ int webbase_insert_url(webbase_t* base, webbase_url_t* webbase_url)
     int has_description = webbase_url->w_info & WEBBASE_URL_INFO_DESCRIPTION;
 
     if(has_relative)
-      sql_quote_char(&relative, &relative_size, webbase_url->w_relative); 
+      sql_quote_char(&relative, &relative_size, webbase_url->w_relative, strlen(webbase_url->w_relative)); 
     if(has_absolute)
-      sql_quote_char(&absolute, &absolute_size, webbase_url->w_absolute); 
+      sql_quote_char(&absolute, &absolute_size, webbase_url->w_absolute, strlen(webbase_url->w_absolute)); 
     static_alloc(&query, &query_size,
-		 query_size +
+		 MINIMUM_QUERY_SIZE +
+		 webbase_url->w_url_length +
+		 webbase_url->w_base_url_length +
+		 webbase_url->w_location_length +
 		 webbase_url->w_relative_length +
 		 webbase_url->w_absolute_length
 		 );
 
 #define S(f,hasf) (hasf ? '\'' : ' '), (hasf ? f : "null"), (hasf ? '\'' : ' ')
-#define T(f,ffield,fsize,hasf) (hasf ? '\'' : ' '), (hasf ? sql_quote_char(&f, &fsize, webbase_url->ffield) : "null"), (hasf ? '\'' : ' ')
+#define T(f,ffield,fsize,hasf) (hasf ? '\'' : ' '), (hasf ? sql_quote_char(&f, &fsize, webbase_url->ffield, strlen(webbase_url->ffield)) : "null"), (hasf ? '\'' : ' ')
 
     sprintf(query, "replace into url_complete values (%d, %c%s%c, %c%s%c, %c%s%c, %c%s%c, %c%s%c, %c%s%c)",
 	    webbase_url->w_complete_rowid,
@@ -821,6 +898,8 @@ int webbase_insert_url(webbase_t* base, webbase_url_t* webbase_url)
   {
     static char* url = 0;
     static int url_size = 0;
+    static char* url_md5 = 0;
+    static int url_md5_size = 0;
     static char* md5 = 0;
     static int md5_size = 0;
     static char* extract = 0;
@@ -830,9 +909,10 @@ int webbase_insert_url(webbase_t* base, webbase_url_t* webbase_url)
     int has_extract = webbase_url->w_info & WEBBASE_URL_INFO_EXTRACT;
     int has_title = webbase_url->w_info & WEBBASE_URL_INFO_TITLE;
 
-    sprintf(query, "replace into url values (%d, '%s', %d, %d, FROM_UNIXTIME(%ld), FROM_UNIXTIME(%ld), %d, '%s', %d, '%s', %d, FROM_UNIXTIME(%ld), %d, '%s', %c%s%c, %c%s%c)",
+    sprintf(query, "replace into url values (%d, '%s', '%s', %d, %d, FROM_UNIXTIME(%ld), FROM_UNIXTIME(%ld), %d, '%s', %d, '%s', %d, FROM_UNIXTIME(%ld), %d, '%s', %c%s%c, %c%s%c)",
 	    webbase_url->w_rowid,
-	    sql_quote_char(&url, &url_size, webbase_url->w_url), 
+	    sql_quote_char(&url, &url_size, webbase_url->w_url, strlen(webbase_url->w_url)),
+	    sql_quote_char(&url_md5, &url_md5_size, webbase_url->w_url_md5, MD5_ASCII_SIZE), 
 	    webbase_url->w_info,
 	    webbase_url->w_code,
 	    (long)webbase_url->w_mtime,
@@ -840,7 +920,7 @@ int webbase_insert_url(webbase_t* base, webbase_url_t* webbase_url)
 	    webbase_url->w_tags,
 	    webbase_url->w_content_type,
 	    webbase_url->w_content_length,
-	    sql_quote_char(&md5, &md5_size, webbase_url->w_md5),
+	    sql_quote_char(&md5, &md5_size, webbase_url->w_md5, MD5_ASCII_SIZE),
 	    complete_rowid,
 	    (long)webbase_url->w_crawl,
 	    webbase_url->w_hookid,
@@ -853,63 +933,33 @@ int webbase_insert_url(webbase_t* base, webbase_url_t* webbase_url)
   }
 #undef S
 #undef T
+#undef MINIMUM_QUERY_SIZE
   return 0;
 }
 
-int webbase_get_url_2(webbase_t* base, char* query, char* url, MYSQL_RES** resp, MYSQL_ROW* rowp)
+webbase_url_t* webbase_get_url(webbase_t* base, unsigned char* url_md5, webbase_url_t* webbase_url, int flag)
 {
-  MYSQL_RES *res;
-  MYSQL_ROW row;
-  int found = 0;
-  
-  /*  if(verbose) fprintf(stderr, "webbase_get_url_1: %s\n", query); */
-  smysql_query(&base->mysql, query);
-  res = smysql_store_result(&base->mysql);
-  /*
-   * Cope with case insensitivity of mysql.
-   */
-  if(mysql_num_rows(res) > 0) {
-    /* if(verbose && mysql_num_rows(res) > 1) fprintf(stderr, "\tcase sensitivity handling for %s\n", url); */
-    while(!found && (row = mysql_fetch_row(res))) {
-      int i;
-      MYSQL_FIELD* field;
-      for(i = 0; !found && i < mysql_num_fields(res); i++) {
-	mysql_field_seek(res, i);
-	field = mysql_fetch_field(res);
-	if(!strcasecmp(field->name, "url") && !strcmp(row[i], url)) {
-	  *rowp = row;
-	  found = 1;
-	}
-      }
-    }
-  }
-
-  *resp = res;
-  return found;
-}
-
-webbase_url_t* webbase_get_url(webbase_t* base, char* url, webbase_url_t* webbase_url, int flag)
-{
-  return webbase_get_url_1(base, "'", "url", url, webbase_url, flag);
+  return webbase_get_url_1(base, "'", "url_md5", url_md5, MD5_ASCII_SIZE, webbase_url, flag);
 }
 
 webbase_url_t* webbase_get_url_rowid(webbase_t* base, int rowid, webbase_url_t* webbase_url, int flag)
 {
-  char rowid_string[32];
+  unsigned char rowid_string[32];
   sprintf(rowid_string, "%d", rowid);
-  return webbase_get_url_1(base, "", "rowid", rowid_string, webbase_url, flag);
+  return webbase_get_url_1(base, "", "rowid", rowid_string, strlen(rowid_string), webbase_url, flag);
 }
 
-webbase_url_t* webbase_get_url_1(webbase_t* base, char* quote, char* field, char* value, webbase_url_t* webbase_url, int flag)
+webbase_url_t* webbase_get_url_1(webbase_t* base, char* quote, char* field, unsigned char* value, int value_length, webbase_url_t* webbase_url, int flag)
 {
   MYSQL_RES *res;
   MYSQL_ROW row;
   static char* query = 0;
   static int query_size = 0;
-  char* quoted_value = sql_quote_char_simple(value);
-  int value_length = strlen(quoted_value);
   int found = 1;
   static webbase_url_t webbase_url_object;
+
+  memset((char*)&row, '\0', sizeof(MYSQL_ROW));
+
   if(!webbase_url) {
     webbase_url = &webbase_url_object;
   }
@@ -918,19 +968,13 @@ webbase_url_t* webbase_get_url_1(webbase_t* base, char* quote, char* field, char
 
   static_alloc(&query, &query_size, 256 + value_length);
 
-  sprintf(query, "select "WEBBASE_URL_FIELDS" from url where %s = %s%s%s", field, quote, quoted_value, quote);
-  if(!strcmp(field, "url")) {
-    if(!webbase_get_url_2(base, query, value, &res, &row)) {
-      found = 0;
-    }
+  sprintf(query, "select "WEBBASE_URL_FIELDS" from url where %s = %s%.*s%s", field, quote, value_length, value, quote);
+  smysql_query(&base->mysql, query);
+  res = smysql_store_result(&base->mysql);
+  if(mysql_num_rows(res) > 0) {
+    row = mysql_fetch_row(res);
   } else {
-    smysql_query(&base->mysql, query);
-    res = smysql_store_result(&base->mysql);
-    if(mysql_num_rows(res) > 0) {
-      row = mysql_fetch_row(res);
-    } else {
-      found = 0;
-    }
+    found = 0;
   }
 
   if(found) {
@@ -968,22 +1012,27 @@ webbase_url_t* webbase_get_url_1(webbase_t* base, char* quote, char* field, char
   return webbase_url;
 }
 
-int webbase_exists_url(webbase_t* base, char* url)
+int webbase_exists_url(webbase_t* base, unsigned char* url_md5)
 {
   MYSQL_RES *res;
   MYSQL_ROW row;
   int rowid = 0;
   static char* query = 0;
   static int query_size = 0;
-  int url_length = strlen(url);
 
-  static_alloc(&query, &query_size, 128 + url_length);
+  static_alloc(&query, &query_size, 128 + MD5_ASCII_SIZE);
 
-  sprintf(query, "select rowid,url from url where url = '%s'", sql_quote_char_simple(url));
-  if(webbase_get_url_2(base, query, url, &res, &row))
+  sprintf(query, "select rowid from url where url_md5 = '%.*s'", MD5_ASCII_SIZE, url_md5);
+
+  smysql_query(&base->mysql, query);
+  res = smysql_store_result(&base->mysql);
+  if(mysql_num_rows(res) > 0) {
+    row = mysql_fetch_row(res);
     rowid = atoi(row[0]);
+  }
 
   mysql_free_result(res);
+
   return rowid;
 }
 
@@ -991,14 +1040,16 @@ int webbase_visited(webbase_t* base, char* url)
 {
   static char* query = 0;
   static int query_size = 0;
-  int url_length = strlen(url);
-  char* quoted_url = sql_quote_char_simple(url);
+  unsigned char url_md5[MD5_ASCII_SIZE];
+  char* quoted_url = sql_quote_char_simple(url, strlen(url));
+  int quoted_url_length = strlen(quoted_url);
   int rowid;
-  static char default_row[] = "insert into url (url, info, code, tags, crawl) values ('%s', 0, 0, 0, 19700101000000)";
+  static char default_row[] = "insert into url (url, url_md5, info, code, tags, crawl) values ('%s', '%.*s', 0, 0, 0, 19700101000000)";
 
-  static_alloc(&query, &query_size, 128 + url_length);
+  static_alloc(&query, &query_size, 256 + MD5_ASCII_SIZE + quoted_url_length);
+  str2md5ascii(url, strlen(url), url_md5);
 
-  sprintf(query, default_row, quoted_url);
+  sprintf(query, default_row, quoted_url, MD5_ASCII_SIZE, url_md5);
   if(verbose > 5) fprintf(stderr, "webbase_visited: %s\n", query);
   smysql_query(&base->mysql, query);
   rowid = mysql_insert_id(&base->mysql);
@@ -1009,6 +1060,10 @@ int webbase_visited(webbase_t* base, char* url)
 static webbase_url_t* webbase_decode_url(webbase_t* base, MYSQL_RES *res, MYSQL_ROW row, webbase_url_t* webbase_url)
 {
   int i;
+  /*
+   * Set if url_md5 field found and not empty.
+   */
+  int url_md5_set = 0;
 
   for(i = 0; i < mysql_num_fields(res); i++) {
     MYSQL_FIELD* field;
@@ -1018,8 +1073,14 @@ static webbase_url_t* webbase_decode_url(webbase_t* base, MYSQL_RES *res, MYSQL_
     if(!strcmp(field->name, "rowid")) {
       webbase_url->w_rowid = atoi(row[i]);
     } else if(!strcmp(field->name, "url")) {
-      strncpy(webbase_url->w_url, row[i], WEBBASE_URL_LENGTH);
-      webbase_url->w_url[WEBBASE_URL_LENGTH] = '\0';
+      int length = strlen(row[i]);
+      static_alloc(&webbase_url->w_url, &webbase_url->w_url_length, length + 1);
+      strcpy(webbase_url->w_url, row[i]);
+    } else if(!strcmp(field->name, "url_md5")) {
+      if(row[i] && field->length == MD5_ASCII_SIZE) {
+	url_md5_set = 1;
+	memcpy(webbase_url->w_url_md5, row[i], MD5_ASCII_SIZE);
+      }
     } else if(!strcmp(field->name, "info+0")) {
       webbase_url->w_info = atoi(row[i]);
     } else if(!strcmp(field->name, "code")) {
@@ -1036,7 +1097,7 @@ static webbase_url_t* webbase_decode_url(webbase_t* base, MYSQL_RES *res, MYSQL_
     } else if(!strcmp(field->name, "content_length")) {
       webbase_url->w_content_length = atoi(row[i]);
     } else if(!strcmp(field->name, "md5")) {
-      memcpy(webbase_url->w_md5, row[i], WEBBASE_MD5_LENGTH);
+      memcpy(webbase_url->w_md5, row[i], MD5_ASCII_SIZE);
     } else if(!strcmp(field->name, "complete_rowid")) {
       webbase_url->w_complete_rowid = atoi(row[i]);
     } else if(!strcmp(field->name, "unix_timestamp(crawl)")) {
@@ -1081,6 +1142,9 @@ static webbase_url_t* webbase_decode_url(webbase_t* base, MYSQL_RES *res, MYSQL_
     }
   }
 
+  if(!url_md5_set)
+    str2md5ascii(webbase_url->w_url, strlen(webbase_url->w_url), webbase_url->w_url_md5);
+
   if(verbose > 1) fprintf(stderr, "decode_url: %s\n", webbase_url->w_url);
   return webbase_url;
 }
@@ -1141,7 +1205,7 @@ int webbase_get_start2url(webbase_t* base, int* start_rowid, int* url_rowid, int
 int webbase_counted(webbase_t* base, webbase_url_start_t* start, int rowid)
 {
   int num_rows = 0;
-  char query[64];
+  char query[128];
   MYSQL_RES *res;
   MYSQL_ROW row;
 
@@ -1163,6 +1227,8 @@ int webbase_counted(webbase_t* base, webbase_url_start_t* start, int rowid)
   }
   mysql_free_result(res);
 
+  if(verbose) fprintf(stderr, "\twebbase_counted: %s -> %d\n", query, num_rows);
+
   return num_rows;
 }
 
