@@ -82,10 +82,13 @@ static struct option long_options[] =
 {
   /* These options set a flag. */
   {"verbose_webtools", 0, &verbose, 1},
+  {"webtools_limit", 1, 0, 0},
   {"http_level", 1, 0, 0},
   {0, 0, 0, WEBTOOLS_OPTIONS}
 };
 
+struct sigaction action_timeout_firewall;
+
 struct option* webtools_options(struct option options[])
 {
   return long_options;
@@ -118,6 +121,9 @@ webtools_params_t* webtools_alloc(int argc, char** argv, struct option options[]
 	if(!strcmp(options[option_index].name, "http_level")) {
 	  params->http_level = strdup(optarg);
 	  break;
+	} else if(!strcmp(options[option_index].name, "webtools_limit")) {
+	  params->size_limit = atoi(optarg);
+	  break;
 	} else if(!strcmp(options[option_index].name, "port")) {
 	  break;
 	}
@@ -135,6 +141,11 @@ webtools_params_t* webtools_alloc(int argc, char** argv, struct option options[]
   return params;
 }
 
+void hand_timeout_firewall(int sig) {
+  
+}
+
+
 void webtools_free(webtools_params_t* params)
 {
   hash_free(params->options);
@@ -210,7 +221,7 @@ int read_timeout(int fd, char* buffer, int buffer_size, int timeout)
     if(ret == 1) {
       return read(fd, buffer, buffer_size);
     } else if(ret == 0) {
-      errno = ETIME;
+      errno = ETIMEDOUT;
       return -1;
     } else {
       return -1;
@@ -258,6 +269,14 @@ static int webtools_reader_http_header(webtools_params_t* params, int sd, struct
       if(!done) {
 	p += bytes_read;
 	buffer_length += bytes_read;
+	/*
+	 * Truncating is not considered as an error. The parser must
+	 * be able to deal with the existing data.
+	 */
+	if(buffer_length > params->size_limit) {
+	  if(verbose) fprintf(stderr, "webtools_reader_http_header: trunc because input > %d (change limit with -webtools_limit <size in bytes> option)", params->size_limit);
+	  done = 1;
+	}
       } else {
 	int total_length = buffer_length + bytes_read;
 	int remain;
@@ -286,6 +305,7 @@ static int webtools_reader_http_body(webtools_params_t* params, int sd, struct w
   file_state_restore
   char* p;
   int code = WEBTOOLS_READER_OK;
+  int total_read = buffer_length;
   int done = 0;
 
   if(verbose) fprintf(stderr, "buffer_length = %d\n", buffer_length);
@@ -305,10 +325,17 @@ static int webtools_reader_http_body(webtools_params_t* params, int sd, struct w
       done = 1;
     } else {
       buffer_length += bytes_read;
+      total_read += bytes_read;
       if(verbose) fprintf(stderr, "buffer_length = %d\n", buffer_length);
       params->callback_http_body(params->callback_arg, sd, buffer, buffer_length, WEBTOOLS_READER_CONTINUE);
       buffer_length = 0;
       p = buffer;
+
+      if(total_read > params->size_limit) {
+	if(verbose) fprintf(stderr, "webtools_reader_http_body: trunc because input > %d (change limit with -webtools_limit <size in bytes> option)", params->size_limit);
+	code = WEBTOOLS_READER_TRUNC;
+	done = 1;
+      }
     }
   }
 
@@ -316,6 +343,8 @@ static int webtools_reader_http_body(webtools_params_t* params, int sd, struct w
   return code;
 }
 
+#define TIMEOUT_FIREWALL 60
+
 static int webtools_open_1(webtools_params_t* params, struct in_addr server, short port)
 {
   struct sockaddr_in sin;
@@ -327,16 +356,21 @@ static int webtools_open_1(webtools_params_t* params, struct in_addr server, sho
     exit(1);
   }
 
+  action_timeout_firewall.sa_handler=hand_timeout_firewall;
+  sigaction(SIGALRM, &action_timeout_firewall, NULL);
+
   memset(&sin, '\0', sizeof(sin));
   sin.sin_family = AF_INET ;
   memcpy((char*)&sin.sin_addr.s_addr, &server, sizeof(struct in_addr));
   sin.sin_port = htons(port);
+  alarm(TIMEOUT_FIREWALL);
   if(connect(sd,(struct sockaddr *) &sin, sizeof(sin)) == -1) {
     switch(errno) {
     case ECONNREFUSED:
     case ETIMEDOUT:
     case EHOSTDOWN:
     case EHOSTUNREACH:
+    case EINTR:
       close(sd);
       return -1;
       break;
@@ -345,6 +379,7 @@ static int webtools_open_1(webtools_params_t* params, struct in_addr server, sho
     perror("");
     exit(1);
   }
+  alarm(0);
 
   if(verbose) fprintf(stderr, "connected\n");
 
@@ -363,6 +398,7 @@ static webtools_params_t* params_alloc()
   memset((char*)params, '\0', sizeof(webtools_params_t));
   params->options = hash_create(33, 0, 0);
   hash_set_allocator(params->options, 0, hnode_free, 0);
+  params->size_limit = 100 * 1024;
 
   return params;
 }
