@@ -31,6 +31,6 @@
 #endif
 
 void time_register(time_t* timevar);
-void time_show(char* message, time_t timevar);
+time_t time_show(char* message, time_t timevar);
 
 #endif /* _timer_h */
